import 'package:test/expect.dart';
import 'package:test/scaffolding.dart';
import 'package:widget_driver_generator/src/utils/package_info_provider.dart';
import 'package:mocktail/mocktail.dart';

class MockFileReader extends Mock implements FileReader {}

void main() {
  late MockFileReader mockFileReader;

  group('PackageInfoProvider:', () {
    setUp(() {
      mockFileReader = MockFileReader();
    });

    test('Returns correct version for pubspec which contains the generator package', () async {
      const expectedVersionNumber = '0.1.2';

      when(() => mockFileReader.readFile(any())).thenAnswer(
        (_) => Future.value(
          _examplePubspecWithGeneratorVersion(expectedVersionNumber),
        ),
      );
      final packageInfoProvider = PackageInfoProvider(fileReader: mockFileReader);
      final pacakgeVersion = await packageInfoProvider.getPackageVersionString();
      expect(pacakgeVersion, expectedVersionNumber);
    });

    test('Returns empty version when pubspec does not contain the generator package', () async {
      when(() => mockFileReader.readFile(any())).thenAnswer(
        (_) => Future.value(_examplePubspecWithoutGeneratorVersion),
      );
      final packageInfoProvider = PackageInfoProvider(fileReader: mockFileReader);
      final pacakgeVersion = await packageInfoProvider.getPackageVersionString();
      expect(pacakgeVersion, '');
    });
  });
}

const _examplePubspecWithoutGeneratorVersion = """
# Generated by pub
# See https://dart.dev/tools/pub/glossary#lockfile
packages:
  _fe_analyzer_shared:
    dependency: transitive
    description:
      name: _fe_analyzer_shared
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "47.0.0"
  analyzer:
    dependency: transitive
    description:
      name: analyzer
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "4.7.0"
""";

// This is an example pubspec.lock which has
// version 0.0.6 for the widget_driver_generator.
String _examplePubspecWithGeneratorVersion(String version) {
  return """
# Generated by pub
# See https://dart.dev/tools/pub/glossary#lockfile
packages:
  _fe_analyzer_shared:
    dependency: transitive
    description:
      name: _fe_analyzer_shared
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "47.0.0"
  analyzer:
    dependency: transitive
    description:
      name: analyzer
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "4.7.0"
  args:
    dependency: transitive
    description:
      name: args
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.3.2"
  async:
    dependency: transitive
    description:
      name: async
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.9.0"
  boolean_selector:
    dependency: transitive
    description:
      name: boolean_selector
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.0"
  build:
    dependency: transitive
    description:
      name: build
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.3.1"
  build_config:
    dependency: transitive
    description:
      name: build_config
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.1.1"
  build_daemon:
    dependency: transitive
    description:
      name: build_daemon
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.1.0"
  build_resolvers:
    dependency: transitive
    description:
      name: build_resolvers
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.10"
  build_runner:
    dependency: "direct dev"
    description:
      name: build_runner
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.3.0"
  build_runner_core:
    dependency: transitive
    description:
      name: build_runner_core
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "7.2.7"
  built_collection:
    dependency: transitive
    description:
      name: built_collection
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "5.1.1"
  built_value:
    dependency: transitive
    description:
      name: built_value
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "8.4.3"
  cached_network_image:
    dependency: "direct main"
    description:
      name: cached_network_image
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.2.3"
  cached_network_image_platform_interface:
    dependency: transitive
    description:
      name: cached_network_image_platform_interface
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.0"
  cached_network_image_web:
    dependency: transitive
    description:
      name: cached_network_image_web
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.2"
  characters:
    dependency: transitive
    description:
      name: characters
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.2.1"
  checked_yaml:
    dependency: transitive
    description:
      name: checked_yaml
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.2"
  clock:
    dependency: transitive
    description:
      name: clock
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.1.1"
  code_builder:
    dependency: transitive
    description:
      name: code_builder
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "4.4.0"
  collection:
    dependency: transitive
    description:
      name: collection
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.16.0"
  convert:
    dependency: transitive
    description:
      name: convert
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.1.1"
  coverage:
    dependency: transitive
    description:
      name: coverage
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.6.2"
  crypto:
    dependency: transitive
    description:
      name: crypto
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.0.2"
  cupertino_icons:
    dependency: "direct main"
    description:
      name: cupertino_icons
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.5"
  dart_style:
    dependency: transitive
    description:
      name: dart_style
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.2.4"
  fake_async:
    dependency: transitive
    description:
      name: fake_async
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.3.1"
  ffi:
    dependency: transitive
    description:
      name: ffi
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.1"
  file:
    dependency: transitive
    description:
      name: file
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "6.1.4"
  fixnum:
    dependency: transitive
    description:
      name: fixnum
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.1"
  flutter:
    dependency: "direct main"
    description: flutter
    source: sdk
    version: "0.0.0"
  flutter_blurhash:
    dependency: transitive
    description:
      name: flutter_blurhash
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.7.0"
  flutter_cache_manager:
    dependency: transitive
    description:
      name: flutter_cache_manager
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.3.0"
  flutter_lints:
    dependency: "direct dev"
    description:
      name: flutter_lints
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.4"
  flutter_test:
    dependency: "direct dev"
    description: flutter
    source: sdk
    version: "0.0.0"
  frontend_server_client:
    dependency: transitive
    description:
      name: frontend_server_client
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.3"
  get_it:
    dependency: "direct main"
    description:
      name: get_it
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "7.2.0"
  glob:
    dependency: transitive
    description:
      name: glob
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.1"
  graphs:
    dependency: transitive
    description:
      name: graphs
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.2.0"
  http:
    dependency: "direct main"
    description:
      name: http
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.13.5"
  http_multi_server:
    dependency: transitive
    description:
      name: http_multi_server
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.2.1"
  http_parser:
    dependency: transitive
    description:
      name: http_parser
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "4.0.2"
  io:
    dependency: transitive
    description:
      name: io
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.4"
  js:
    dependency: transitive
    description:
      name: js
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.6.5"
  json_annotation:
    dependency: transitive
    description:
      name: json_annotation
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "4.8.0"
  lints:
    dependency: transitive
    description:
      name: lints
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.1"
  logging:
    dependency: transitive
    description:
      name: logging
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.1.0"
  matcher:
    dependency: transitive
    description:
      name: matcher
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.12.12"
  material_color_utilities:
    dependency: transitive
    description:
      name: material_color_utilities
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.1.5"
  meta:
    dependency: "direct main"
    description:
      name: meta
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.8.0"
  mime:
    dependency: transitive
    description:
      name: mime
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.4"
  mocktail:
    dependency: "direct dev"
    description:
      name: mocktail
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.1.4"
  nested:
    dependency: transitive
    description:
      name: nested
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.0"
  node_preamble:
    dependency: transitive
    description:
      name: node_preamble
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.1"
  octo_image:
    dependency: transitive
    description:
      name: octo_image
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.2"
  package_config:
    dependency: transitive
    description:
      name: package_config
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.0"
  path:
    dependency: transitive
    description:
      name: path
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.8.2"
  path_provider:
    dependency: transitive
    description:
      name: path_provider
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.11"
  path_provider_android:
    dependency: transitive
    description:
      name: path_provider_android
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.22"
  path_provider_ios:
    dependency: transitive
    description:
      name: path_provider_ios
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.11"
  path_provider_linux:
    dependency: transitive
    description:
      name: path_provider_linux
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.7"
  path_provider_macos:
    dependency: transitive
    description:
      name: path_provider_macos
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.7"
  path_provider_platform_interface:
    dependency: transitive
    description:
      name: path_provider_platform_interface
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.0.5"
  path_provider_windows:
    dependency: transitive
    description:
      name: path_provider_windows
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.3"
  pedantic:
    dependency: transitive
    description:
      name: pedantic
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.11.1"
  platform:
    dependency: transitive
    description:
      name: platform
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.1.0"
  plugin_platform_interface:
    dependency: transitive
    description:
      name: plugin_platform_interface
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.3"
  pool:
    dependency: transitive
    description:
      name: pool
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.5.1"
  process:
    dependency: transitive
    description:
      name: process
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "4.2.4"
  provider:
    dependency: "direct main"
    description:
      name: provider
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "6.0.5"
  pub_semver:
    dependency: transitive
    description:
      name: pub_semver
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.3"
  pubspec_parse:
    dependency: transitive
    description:
      name: pubspec_parse
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.2.1"
  rxdart:
    dependency: transitive
    description:
      name: rxdart
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.27.7"
  shelf:
    dependency: transitive
    description:
      name: shelf
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.4.0"
  shelf_packages_handler:
    dependency: transitive
    description:
      name: shelf_packages_handler
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.0.1"
  shelf_static:
    dependency: transitive
    description:
      name: shelf_static
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.1.1"
  shelf_web_socket:
    dependency: transitive
    description:
      name: shelf_web_socket
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.3"
  sky_engine:
    dependency: transitive
    description: flutter
    source: sdk
    version: "0.0.99"
  source_gen:
    dependency: transitive
    description:
      name: source_gen
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.2.6"
  source_map_stack_trace:
    dependency: transitive
    description:
      name: source_map_stack_trace
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.1"
  source_maps:
    dependency: transitive
    description:
      name: source_maps
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.10.11"
  source_span:
    dependency: transitive
    description:
      name: source_span
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.9.0"
  sqflite:
    dependency: transitive
    description:
      name: sqflite
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.2.3"
  sqflite_common:
    dependency: transitive
    description:
      name: sqflite_common
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.4.1"
  stack_trace:
    dependency: transitive
    description:
      name: stack_trace
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.10.0"
  stream_channel:
    dependency: transitive
    description:
      name: stream_channel
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.0"
  stream_transform:
    dependency: transitive
    description:
      name: stream_transform
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.0"
  string_scanner:
    dependency: transitive
    description:
      name: string_scanner
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.1.1"
  synchronized:
    dependency: transitive
    description:
      name: synchronized
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.0.1"
  term_glyph:
    dependency: transitive
    description:
      name: term_glyph
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.2.1"
  test:
    dependency: transitive
    description:
      name: test
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.21.4"
  test_api:
    dependency: transitive
    description:
      name: test_api
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.4.12"
  test_core:
    dependency: transitive
    description:
      name: test_core
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.4.16"
  timing:
    dependency: transitive
    description:
      name: timing
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.1"
  typed_data:
    dependency: transitive
    description:
      name: typed_data
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.3.1"
  uuid:
    dependency: transitive
    description:
      name: uuid
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.0.7"
  vector_math:
    dependency: transitive
    description:
      name: vector_math
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.1.2"
  vm_service:
    dependency: transitive
    description:
      name: vm_service
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "9.4.0"
  watcher:
    dependency: transitive
    description:
      name: watcher
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.0.2"
  web_socket_channel:
    dependency: transitive
    description:
      name: web_socket_channel
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "2.3.0"
  webkit_inspection_protocol:
    dependency: transitive
    description:
      name: webkit_inspection_protocol
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "1.2.0"
  widget_driver:
    dependency: "direct main"
    description:
      path: ".."
      relative: true
    source: path
    version: "0.0.8"
  widget_driver_annotation:
    dependency: transitive
    description:
      path: "../../widget_driver_annotation"
      relative: true
    source: path
    version: "0.0.4"
  widget_driver_generator:
    dependency: "direct dev"
    description:
      path: "../../widget_driver_generator"
      relative: true
    source: path
    version: $version
  widget_driver_test:
    dependency: "direct dev"
    description:
      path: "../../widget_driver_test"
      relative: true
    source: path
    version: "0.0.5"
  win32:
    dependency: transitive
    description:
      name: win32
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.1.3"
  xdg_directories:
    dependency: transitive
    description:
      name: xdg_directories
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "0.2.0+3"
  yaml:
    dependency: transitive
    description:
      name: yaml
      url: "https://mobile20unpubfrontdoor.azurefd.net"
    source: hosted
    version: "3.1.1"
sdks:
  dart: ">=2.18.0 <3.0.0"
  flutter: ">=3.3.0"

""";
}
